import base64
import json
import os

from requests import Session
from requests.adapters import HTTPAdapter
from requests.exceptions import HTTPError
from requests.packages.urllib3.util.retry import Retry

from abeja.common.auth import get_credential
from abeja.exceptions import (
    BadRequest,
    Unauthorized,
    Forbidden,
    NotFound,
    MethodNotAllowed,
    Conflict,
    InternalServerError
)

MAX_RETRY_COUNT = 5
CONNECTION_TIMEOUT = 30


class Connection:
    """A connection to ABEJA Platform API."""
    BASE_URL = os.environ.get('ABEJA_API_URL', 'https://api.abeja.io')

    def __init__(self, credential=None):
        if credential is None:
            self.credential = get_credential() or {}
        else:
            self.credential = credential
        self._session = self._generate_session()

    def api_request(self, method, path, data=None, json=None, headers=None, params=None, **kwargs):
        """call platform api and handle errors if needed

        :param method:
        :param path:
        :param data:
        :param headers:
        :param params:
        :param json:
        :param kwargs:
        :return: (dict) api response
        """
        if headers is None:
            headers = {}
        headers.update(self._get_auth_header())
        try:
            res = self.request(method,
                               '{}{}'.format(self.BASE_URL, path),
                               data=data,
                               json=json,
                               headers=headers,
                               params=params,
                               **kwargs)
            return res.json()
        except HTTPError as e:
            api_error_handler(e)

    def request(self, method, url, data=None, json=None, headers=None, params=None, **kwargs):
        """make request with retry and timeout settings.

        :param method:
        :param url:
        :param data:
        :param json:
        :param headers:
        :param params:
        :param kwargs:
        :return: (Response)
        """
        res = self._session.request(method, url, data=data, json=json, params=params,
                                    headers=headers, timeout=CONNECTION_TIMEOUT,
                                    **kwargs)
        res.raise_for_status()
        return res

    def _generate_session(self):
        """generate simple session to retry
        :return: session
        """
        session = Session()
        retries = Retry(total=MAX_RETRY_COUNT,
                        backoff_factor=1,
                        method_whitelist=('GET', 'POST', 'PUT', 'PATCH', 'DELETE'),
                        status_forcelist=(500, 502, 503, 504),
                        raise_on_status=False)
        session.mount('http://', HTTPAdapter(max_retries=retries))
        session.mount('https://', HTTPAdapter(max_retries=retries))
        return session

    def _get_auth_header(self):
        if self.credential.get('auth_token'):
            return {
                'Authorization': 'Bearer {}'.format(self.credential['auth_token'])
            }
        if self.credential.get('user_id') and self.credential.get('personal_access_token'):
            user_id = self.credential['user_id']
            personal_access_token = self.credential['personal_access_token']
            base = '{}:{}'.format(user_id, personal_access_token)
            encoded = base64.b64encode(base.encode('utf-8'))
            return {
                'Authorization': 'Basic {}'.format(encoded.decode('utf-8'))
            }
        return {}


def api_error_handler(e):
    try:
        res = e.response.json()
    except json.JSONDecodeError:
        raise e
    if 'error' not in res or 'error_description' not in res:
        raise e
    if e.response.status_code == 400:
        raise BadRequest(res['error'], res['error_description'])
    if e.response.status_code == 401:
        raise Unauthorized(res['error'], res['error_description'])
    if e.response.status_code == 403:
        raise Forbidden(res['error'], res['error_description'])
    if e.response.status_code == 404:
        raise NotFound(res['error'], res['error_description'])
    if e.response.status_code == 405:
        raise MethodNotAllowed(res['error'], res['error_description'])
    if e.response.status_code == 409:
        raise Conflict(res['error'], res['error_description'])
    if e.response.status_code == 500:
        raise InternalServerError(res['error'], res['error_description'])
    raise e


def http_error_handler(e):
    if e.response.status_code == 400:
        raise BadRequest(e.response._content)
    if e.response.status_code == 401:
        raise Unauthorized(e.response._content)
    if e.response.status_code == 403:
        raise Forbidden(e.response._content)
    if e.response.status_code == 404:
        raise NotFound(e.response._content)
    if e.response.status_code == 405:
        raise MethodNotAllowed(e.response._content)
    if e.response.status_code == 409:
        raise Conflict(e.response._content)
    if e.response.status_code == 500:
        raise InternalServerError(e.response._content)
    raise e
