# -*- coding: utf-8 -*-
import os
import mimetypes
from typing import Iterable
from concurrent.futures import ThreadPoolExecutor, as_completed

from abeja.common.config import UPLOAD_WORKER_COUNT
from abeja.common.logging import logger
from abeja.common.file_helpers import generate_path_iter
from .api.client import APIClient
from .file import DatalakeFile, Files, FileIterator


class Channel:
    """a model class for a channel

    Properties:
        - organization_id (str)
        - channel_id (str)
        - name (str)
        - display_name (str)
        - description (str)
        - archived (bool)
        - created_at (datetime)
        - updated_at (datetime)

    """

    def __init__(self, api: APIClient, organization_id: str, channel_id: str,
                 name: str=None, description: str=None, display_name: str=None,
                 storage_type: str=None, created_at: str=None,
                 updated_at: str=None, archived: bool=False):
        self._api = api
        self.organization_id = organization_id
        self.channel_id = channel_id
        self.name = name
        self.description = description
        self.display_name = display_name
        self.storage_type = storage_type
        self.created_at = created_at
        self.updated_at = updated_at
        self.archived = archived

    @property
    def files(self) -> Files:
        """Get datalake Files object

        Request syntax:
            .. code-block:: python

                channel = client.get_channel(channel_id='1230000000000')
                files = channel.files()

        Returns:
            :meth:`Files <abeja.channel.file.Files>`
        """
        return Files(self._api, self.organization_id, self.channel_id)

    def list_files(self, start: str=None, end: str=None, timezone: str=None,
                   next_page_token: str=None, limit: int=None, prefetch: bool=False) -> FileIterator:
        """get datalake files in the channel

        Request syntax:
            .. code-block:: python

                for f in channel.get_files():
                    pass

        Params:
            - **start** (str): start date of target uploaded files
            - **end** (str): end date of target uploaded files
            - **timezone** (str): timezone of specified start and end date

        Return type:
            FileIterator
        """
        return FileIterator(self._api, self.organization_id, self.channel_id,
                            start=start, end=end, timezone=timezone, next_page_token=next_page_token,
                            items_per_page=limit, prefetch=prefetch)

    def upload_file(self, file_path: str, metadata: dict=None, content_type: str=None) -> DatalakeFile:
        """upload a file to a channel.
        This method infers the content_type of given file if content_type is not specified,
        and set the filename as `x-abeja-meta-filename` in metadata.

        Request syntax:
            .. code-block:: python

                metadata = {
                    'label': 'example'
                }
                with open('example.csv') as f:
                    response = channel.upload(f, metadata)

        Params:
            - **content** (file-like object) : contents to be uploaded
            - **metadata** (dict): metadata to be added to uploaed file. **[optional]**
            - **content_type** (str): MIME type of content. Content-Type is assumed by the extension if not specified **[optional]**

        Return type:
            DatalakeFile

        Returns:
            a file uploaded to a channel
        """
        if not content_type:
            mime_type, _ = mimetypes.guess_type(file_path)
            content_type = mime_type

        if not metadata:
            metadata = {}
        # add `x-abeja-meta-filename` if not defined
        if 'x-abeja-meta-filename' not in metadata:
            metadata['x-abeja-meta-filename'] = os.path.basename(file_path)
        if 'Content-Type' in metadata:
            # ignore Content-Type in metadata
            metadata.pop('Content-Type')

        upload_info = self._api.get_channel_file_upload(
            self.channel_id, content_type=content_type, metadata=metadata)

        headers = {}
        if content_type:
            headers['Content-Type'] = content_type
        headers.update(metadata)

        with open(file_path, 'rb') as f:
            res = self._api._connection.request('PUT', upload_info['upload_url'],
                                                data=f.read(), headers=headers)
        res.raise_for_status()
        return DatalakeFile(self._api, self.organization_id, self.channel_id, **upload_info)

    def upload_dir(self, dir_path: str, metadata: dict=None, content_type: str=None,
                   recursive: bool=False, use_thread: bool=True) -> Iterable[DatalakeFile]:
        """upload files in directory to a channel.
        This method infers the content_type of given file if content_type is not specified,
        and set the filename as `x-abeja-meta-filename` in metadata.

        Note: this method returns list ( not generator ) to make sure upload process will be done here.

        Request syntax:
            .. code-block:: python

                metadata = {
                    'label': 'example'
                }
                response = channel.upload_dir('./source_dir', metadata)

        Params:
            - **content** (file-like object) : contents to be uploaded
            - **metadata** (dict): metadata to be added to uploaed file. **[optional]**
            - **content_type** (str): MIME type of content. Content-Type is assumed by extensions if not specified **[optional]**

        Return type:
            list(DatalakeFile)

        Returns:
            A list of DatalakeFile successfully uploaded.
        """
        file_path_iter = generate_path_iter(dir_path, recursive=recursive)
        if use_thread:
            upload_files_func = self._upload_files_threaded
        else:
            upload_files_func = self._upload_files_unthreaded
        return upload_files_func(
            file_path_iter, content_type=content_type, metadata=metadata)

    def _upload_files_threaded(
            self, file_paths: Iterable[str], content_type: str=None,
            metadata: dict=None) -> Iterable[DatalakeFile]:
        """upload files asynchronously using thread
        this method does not return generator to avoid lazy evaluation.
        """
        files = []
        with ThreadPoolExecutor(max_workers=UPLOAD_WORKER_COUNT) as executor:
            futures = []
            for f in file_paths:
                futures.append(
                    executor.submit(self.upload_file, f, metadata, content_type))
            for f in as_completed(futures):
                try:
                    files.append(f.result())
                except Exception as e:
                    logger.error(e)
        return files

    def _upload_files_unthreaded(self, file_paths: Iterable[str], content_type: str=None,
                                 metadata: dict=None) -> Iterable[DatalakeFile]:
        """upload files synchronously using thread
        this method does not return generator to avoid lazy evaluation.
        """
        files = []
        for file_path in file_paths:
            try:
                file = self.upload_file(
                    file_path, content_type=content_type, metadata=metadata)
                files.append(file)
            except Exception as e:
                logger.error(e)
        return files

    def list_datasources(self):
        raise NotImplementedError

    def add_datasource(self):
        raise NotImplementedError

    def remove_datasource(self):
        raise NotImplementedError


class Channels:
    """a class for handling channels"""
    def __init__(self, api: APIClient, organization_id: str):
        self._api = api
        self.organization_id = organization_id

    def create(self, name: str, description: str, storage_type: str) -> Channel:
        """create a channel

        API reference: POST /organizations/<organization_id>/channels/

        Request Syntax:
            .. code-block:: python

                params = {
                    "name": "test-channel",
                    "description": "test channel",
                    "storage_type": "datalake"
                }
                channel = channels.create(**params)

            Params:
                - **name** (str): channel name
                - **description** (str): channel description
                - **storage_type** (str): type of storage, datalake or file

            Return type:
                Channel

        """
        res = self._api.create_channel(
            self.organization_id, name, description, storage_type)
        return Channel(self._api, organization_id=self.organization_id,
                       **res['channel'])

    def list(self, limit: int=None, offset: int=None) -> Iterable[Channel]:
        """list channels

        API reference: GET /organizations/<organization_id>/channels/

        Request Syntax:
            .. code-block:: python

                channel = channels.list()

            Return type:
                list

        """
        res = self._api.list_channels(self.organization_id, limit=limit, offset=offset)
        for item in res['channels']:
            yield Channel(self._api, organization_id=self.organization_id, **item)

    def get(self, channel_id: str) -> Channel:
        """get a channel

        API reference: GET /organizations/<organization_id>/channels/<channel_id>

        Request Syntax:
            .. code-block:: python

                channel_id = '1234567890123'
                channel = channels.get(channel_id=channel_id)

            Params:
                - **channel_id** (str): identifier of channel

            Return type:
                Channel

        """
        res = self._api.get_channel(self.organization_id, channel_id)
        return Channel(self._api, organization_id=self.organization_id, **res['channel'])

    def patch(self, channel_id: str, name: str=None, description: str=None) -> Channel:
        """patch a channel

        API reference: PATCH /organizations/<organization_id>/channels/<channel_id>

        Request Syntax:
            .. code-block:: python

                params = {
                    "channel_id": "1234567890123",
                    "name": "updated_name",
                    "description": "updated description"
                }
                channel = channels.patch(**params)

            Params:
                - **channel_id** (str): identifier of channel
                - **name** (str): channel name
                - **description** (str): channel description

            Return type:
                Channel

        """
        res = self._api.patch_channel(
            self.organization_id, channel_id, name, description)
        return Channel(self._api, organization_id=self.organization_id, **res['channel'])
