# -*- coding: utf-8 -*-
from abeja.common.api_client import BaseAPIClient


class APIClient(BaseAPIClient):
    """A low-level client for Dataset API

    .. code-block:: python

       from abeja.datasets import APIClient

       api_client = APIClient()
    """

    def __init__(self, credential=None):
        super().__init__(credential=credential)

    # dataset
    def create_dataset(self, organization_id, name, type, props):
        """create a dataset

        API reference: POST /organizations/<organization_id>/datasets/

        Request Syntax:
            .. code-block:: python

                organization_id = "1102940376065"
                dataset_name = "test-dataset"
                dataset_type = "detection"
                props = {
                    "attributes": [
                        {
                            "categories": [
                                {
                                    "id": 1,
                                    "name": "犬"
                                },
                                {
                                    "id": 2,
                                    "name": "猫"
                                },
                                {
                                    "id": 3,
                                    "name": "その他"
                                }
                            ],
                            "id": 0,
                            "name": "cats_dogs"
                        }
                    ]
                }
                response = api_client.create_dataset(organization_id, dataset_name, dataset_type, props)

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **name** (str): dataset name
            - **type** (str): dataset types eg: classification, detection
            - **props** (dict): properties of dataset

        Return type:
            dict
        Returns:
            Response Syntax:

            .. code-block:: json

                {
                    "created_at": "2018-04-10T07:49:30.514794",
                    "dataset_id": "1410805969256",
                    "name": "test-dataset",
                    "organization_id": "1102940376065",
                    "props": {
                        "attributes": [
                            {
                                "categories": [
                                    {
                                        "id": 1,
                                        "name": "犬"
                                    },
                                    {
                                        "id": 2,
                                        "name": "猫"
                                    },
                                    {
                                        "id": 3,
                                        "name": "その他"
                                    }
                                ],
                                "id": 0,
                                "name": "cats_dogs"
                            }
                        ]
                    },
                    "type": "detection",
                    "updated_at": "2018-04-10T07:49:30.514794"
                }

        Raises:
            - BadRequest: the resource already exists or parameters is insufficient or invalid.
            - Unauthorized: Authentication failed
            - InternalServerError
        """
        params = {
            'name': name,
            'type': type,
            'props': props
        }
        path = '/organizations/{}/datasets'.format(organization_id)
        return self._connection.api_request(method='POST', path=path, params=params)

    def get_dataset(self, organization_id, dataset_id):
        """get a dataset

        API reference: GET /organizations/<organization_id>/datasets/<dataset_id>

        Request Syntax:
            .. code-block:: python

                response = api_client.get_dataset(organization_id='1102940376065', dataset_id='1410805969256')
        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **dataset_id** (str): dataset_id of the requested dataset

        Return type:
            dict

        Returns:
            Response Syntax:
                .. code-block:: json

                    {
                        "created_at": "2018-04-10T07:49:30.514794",
                        "dataset_id": "1410805969256",
                        "name": "test-dataset",
                        "organization_id": "1102940376065",
                        "props": {
                            "attributes": [
                                {
                                    "categories": [
                                        {
                                            "id": 1,
                                            "name": "犬"
                                        },
                                        {
                                            "id": 2,
                                            "name": "猫"
                                        },
                                        {
                                            "id": 3,
                                            "name": "その他"
                                        }
                                    ],
                                    "id": 0,
                                    "name": "cats_dogs"
                                }
                            ]
                        },
                        "type": "detection",
                        "updated_at": "2018-04-10T07:49:30.514794"
                    }

        Raises:
          - NotFound: dataset not found
          - Unauthorized: Authentication failed
          - InternalServerError
        """

        path = '/organizations/{}/datasets/{}'.format(organization_id, dataset_id)
        return self._connection.api_request(method='GET', path=path)

    def list_datasets(self, organization_id, max_results=None, next_token=None):
        """Get datasets list

        API reference: GET /organizations/<organization_id>/datasets/

        Request syntax:
            .. code-block:: python

                response = api_client.get_dataset(organization_id='1102940376065')

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **max_results** (int): maximum number of datasets in case of passing **[optional]**
            - **next_token** (str): To get the next list of datasets **[optional]**

        Return type:
            list

        Returns:
            Return syntax:
                .. code-block:: python

                    [
                        {
                            "created_at": "2018-03-03T09:04:58.274324",
                            "dataset_id": "1377232365920",
                            "name": "cats-dogs",
                            "organization_id": "1102940376065",
                            "props": {
                                "attributes": [
                                    {
                                        "categories": [
                                            {
                                                "id": 1,
                                                "name": "犬"
                                            },
                                            {
                                                "id": 2,
                                                "name": "猫"
                                            },
                                            {
                                                "id": 3,
                                                "name": "その他"
                                            }
                                        ],
                                        "id": 0,
                                        "name": "cats_dogs"
                                    }
                                ]
                            },
                            "type": "classify",
                            "updated_at": "2018-03-03T09:04:58.274324"
                        },
                        {
                            .....
                            .....
                        },
                        .....
                    ]


        Raises:
          - Unauthorized: Authentication failed
          - InternalServerError
        """
        params = {}
        if max_results:
            params['max_results'] = max_results
        if next_token:
            params['next_token'] = next_token
        path = '/organizations/{}/datasets'.format(organization_id)
        return self._connection.api_request(method='GET', path=path, params=params)

    def delete_dataset(self, organization_id, dataset_id):
        """delete a dataset

        API reference: DELETE /organizations/<organization_id>/datasets/<dataset_id>

        Request syntax:
            .. code-block:: python

                response = api_client.delete_dataset(organization_id='1102940376065', dataset_id='1410805969256')

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **dataset_id** (str): dataset id

        Return type:
            dict

        Responses:
            Response syntax:
                .. code-block:: python

                    {
                        "created_at": "2018-04-10T07:49:30.514794",
                        "dataset_id": "1410805969256",
                        "name": "test-dataset",
                        "organization_id": "1102940376065",
                        "props": {
                            "attributes": [
                                {
                                    "categories": [
                                        {
                                            "id": 1,
                                            "name": "犬"
                                        },
                                        {
                                            "id": 2,
                                            "name": "猫"
                                        },
                                        {
                                            "id": 3,
                                            "name": "その他"
                                        }
                                    ],
                                    "id": 0,
                                    "name": "cats_dogs"
                                }
                            ]
                        },
                        "type": "detection",
                        "updated_at": "2018-04-10T07:49:30.514794"
                    }

        Raises:
          - NotFound: dataset not found
          - Unauthorized: Authentication failed
          - InternalServerError
        """
        path = '/organizations/{}/datasets/{}'.format(organization_id, dataset_id)
        return self._connection.api_request(method='DELETE', path=path)

    # dataset_item
    def create_dataset_item(self, organization_id, dataset_id,
                            source_data, attributes=None):
        """create a item in a dataset

        API reference: POST /organizations/<organization_id>/datasets/<dataset_id>/items/

        Request syntax:
            .. code-block:: python

                source_data = [
                    {
                        'data_type': 'images/jpeg',
                        'data_uri': 'datalake://1200123803688/20170815T044617-f20dde80-1e3b-4496-bc06-1b63b026b872',
                        'height': 500,
                        'width': 200
                    }
                ]

                attributes = {
                    'classification': {
                        'id': 1,
                        'label': '犬'
                    },
                    'custom_format': {
                        'anything': 'something'
                    },
                    'detection': [
                        {
                            'id': 2,
                            'label': '猫',
                            'rect': [795, 118, 1143, 418]
                        },
                    ]
                }
                response = api_client.create_dataset_item(
                    organization_id='1102940376065',
                    dataset_id='1410805969256',
                    source_data=source_data,
                    attributes=attributes
                )

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **dataset_id** (str): dataset id
            - **source_data** (list): list of source data
            - **attributes** : attribute of dataset item **[optional]**

        Return type:
            list
        Returns:
            Return syntax:
                .. code-block:: python

                    {
                        'attributes': {
                            'classification': {
                                'id': 1,
                                'label': '犬'
                            },
                            'custom_format': {
                                'anything': 'something'
                            },
                            'detection': [
                                {
                                    'id': 2,
                                    'label': '猫',
                                    'rect': [795, 118, 1143, 418]
                                },
                            ]
                        },
                        'created_at': '2017-12-27T06:25:00.394026',
                        'dataset_id': '1410805969256',
                        'dataset_item_id': 0,
                        'source_data': [
                            {
                                'data_type': 'images/jpeg',
                                'data_uri': 'datalake://1200123803688/20170815T044617-f20dde80-1e3b-4496-bc06-1b63b026b872',
                                'height': 500,
                                'width': 200
                            }
                        ],
                        'organization_id': '1102940376065',
                        'updated_at': '2017-12-27T06:25:00.394026'
                    }
        Raises:
          - BadRequest: specified dataset id does not exist or dataset item id already exist,
                        parameters is insufficient or invalid,
                        input data exceeded the max limit
          - Unauthorized: Authentication failed
          - InternalServerError
        """
        params = {
            'source_data': source_data,
            'attributes': attributes
        }
        path = '/organizations/{}/datasets/{}/items'.format(organization_id,
                                                            dataset_id)
        return self._connection.api_request(method='POST',
                                            path=path,
                                            params=params)

    def get_dataset_item(self, organization_id, dataset_id, dataset_item_id):
        """get a item in a dataset

        API reference: GET /organizations/<organization_id>/datasets/<dataset_id>/items/<dataset_item_id>

        Request syntax:
            .. code-block:: python

                response = api_client.get_dataset_item(organization_id='1102940376065',
                                                        dataset_id='1410805969256',
                                                        dataset_item_id=0
                                                    )


        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **dataset_id** (str): dataset id
            - **dataset_item_id** (int): dataset item id

        Return type:
            dict

        Returns:
            Return syntax:
                .. code-block:: python

                    {
                        'attributes': {
                            'classification': {
                                'id': 1,
                                'label': '犬'
                            },
                            'custom_format': {
                                'anything': 'something'
                            },
                            'detection': [
                                {
                                    'id': 2,
                                    'label': '猫',
                                    'rect': [795, 118, 1143, 418]
                                },
                            ]
                        },
                        'created_at': '2017-12-27T06:25:00.394026',
                        'dataset_id': '1410805969256',
                        'dataset_item_id': 0,
                        'source_data': [
                            {
                                'data_type': 'images/jpeg',
                                'data_uri': 'datalake://1200123803688/20170815T044617-f20dde80-1e3b-4496-bc06-1b63b026b872',
                                'height': 500,
                                'width': 200
                            }
                        ],
                        'organization_id': '1102940376065',
                        'updated_at': '2017-12-27T06:25:00.394026'
                    }

        Raises:
          - NotFound: dataset not found
          - Unauthorized: Authentication failed
          - InternalServerError
        """
        path = '/organizations/{}/datasets/{}/items/{}'.format(organization_id,
                                                               dataset_id,
                                                               dataset_item_id)
        return self._connection.api_request(method='GET', path=path)

    def list_dataset_items(self, organization_id, dataset_id, params=None):
        """Get item list in a dataset

        API reference: GET /organizations/<organization_id>/datasets/<dataset_id>/items/

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **dataset_id** (str): dataset id
            - **params** (dict): **[optional]**

        Return type:
            list

        Returns:
            dataset item list

        Raises:
          - NotFound: dataset not found
          - Unauthorized: Authentication failed
          - InternalServerError
        """
        path = '/organizations/{}/datasets/{}/items'.format(organization_id,
                                                            dataset_id)
        if not params:
            params = {}
        return self._connection.api_request(method='GET', path=path, params=params)

    def delete_dataset_item(self, organization_id, dataset_id, dataset_item_id):
        """delete a item in a dataset

        API reference: DELETE /organizations/<organization_id>/datasets/<dataset_id>/items/<dataset_item_id>

        Request syntax:
            .. code-block:: python

                api_client.delete_dataset_item(organization_id='1102940376065',
                                                        dataset_id='1410805969256',
                                                        dataset_item_id=0
                                                    )

        Params:
            - **organization_id** (str): ORGANIZATION_ID
            - **dataset_id** (str): dataset id
            - **dataset_item_id** (str): dataset item id

        Return type:
            dict

        Returns:
            deleted dataset item

        Raises:
          - NotFound: dataset not found
          - Unauthorized: Authentication failed
          - InternalServerError
        """
        path = '/organizations/{}/datasets/{}/items/{}'.format(organization_id,
                                                               dataset_id,
                                                               dataset_item_id)
        return self._connection.api_request(method='DELETE', path=path)
