# -*- coding: utf-8 -*-
from abeja.datasets.base import DatasetBase
from abeja.datasets.dataset_item import DatasetItems


class Dataset(DatasetBase):
    """a model class for a dataset

    Properties:
        - organization_id (str)
        - dataset_id (str)
        - name (str)
        - type (str)
        - props (dict)
        - created_at (datetime)
        - updated_at (datetime)

    """

    def __init__(self, api, organization_id, dataset_id,
                 name=None, type=None, props=None,
                 created_at=None, updated_at=None, **kwargs):
        self._api = api
        self.organization_id = organization_id
        self.dataset_id = dataset_id
        self.name = name
        self.type = type
        self.props = props
        self.created_at = created_at
        self.updated_at = updated_at

    @property
    def dataset_items(self):
        """Get dataset Items object

                Request syntax:
                    .. code-block:: python

                        dataset = client.get_dataset(dataset_id='1410805969256')
                        dataset_items = dataset.dataset_items()

                Returns:
                    :meth:`DatasetItem <abeja.datasets.dataset_item.DatasetItem>`
        """
        return DatasetItems(self._api, self.organization_id, self.dataset_id)

    def __repr__(self):
        return "<{} organization_id:{} " \
               "dataset_id:{} name:{} type:{} " \
               "props:{} created_at:{} updated_at:{}".format(self.__class__.__name__,
                                                             self.organization_id,
                                                             self.dataset_id,
                                                             self.name,
                                                             self.type,
                                                             self.props,
                                                             self.created_at,
                                                             self.updated_at)


class Datasets:
    """a class for handling datasets"""

    def __init__(self, api, organization_id):
        self._api = api
        self.organization_id = organization_id

    def create(self, name, type, props):
        """create a dataset

        API reference: POST /organizations/<organization_id>/datasets/

        Request Syntax:
            .. code-block:: python

                params = {
                    "name": "test-dataset",
                    "type": "detection",
                    "props": {
                        "attributes": [
                            {
                                "categories": [
                                    {
                                        "id": 1,
                                        "name": "犬"
                                    },
                                    {
                                        "id": 2,
                                        "name": "猫"
                                    },
                                    {
                                        "id": 3,
                                        "name": "その他"
                                    }
                                ],
                                "id": 0,
                                "name": "cats_dogs"
                            }
                        ]
                    }
                }
                response = datasets.create(params)

        Params:
            - **name** (str): dataset name
            - **type** (str): dataset types eg: classification, detection
            - **props** (dict): properties of dataset

        Return type:
            dict
        Returns:
            Response Syntax:

            .. code-block:: json

                {
                    "created_at": "2018-04-10T07:49:30.514794",
                    "dataset_id": "1410805969256",
                    "name": "test-dataset",
                    "organization_id": "1102940376065",
                    "props": {
                        "attributes": [
                            {
                                "categories": [
                                    {
                                        "id": 1,
                                        "name": "犬"
                                    },
                                    {
                                        "id": 2,
                                        "name": "猫"
                                    },
                                    {
                                        "id": 3,
                                        "name": "その他"
                                    }
                                ],
                                "id": 0,
                                "name": "cats_dogs"
                            }
                        ]
                    },
                    "type": "detection",
                    "updated_at": "2018-04-10T07:49:30.514794"
                }
        """
        res = self._api.create_dataset(self.organization_id,
                                       name, type, props)
        return Dataset(self._api, **res)

    def get(self, dataset_id):
        """get a dataset

        Request syntax:
            .. code-block:: python

                response = datasets.get(dataset_id=1410805969256)

        Params:
            - **dataset_id** (str): dataset id

        Return type:
            dict
        Returns:
            Response Syntax:

            .. code-block:: json

                {
                    "created_at": "2018-04-10T07:49:30.514794",
                    "dataset_id": "1410805969256",
                    "name": "test-dataset",
                    "organization_id": "1102940376065",
                    "props": {
                        "attributes": [
                            {
                                "categories": [
                                    {
                                        "id": 1,
                                        "name": "犬"
                                    },
                                    {
                                        "id": 2,
                                        "name": "猫"
                                    },
                                    {
                                        "id": 3,
                                        "name": "その他"
                                    }
                                ],
                                "id": 0,
                                "name": "cats_dogs"
                            }
                        ]
                    },
                    "type": "detection",
                    "updated_at": "2018-04-10T07:49:30.514794"
                }

        """
        res = self._api.get_dataset(self.organization_id, dataset_id)
        return Dataset(self._api, **res)

    def list(self):
        """Get dataset list

        Request syntax:
            .. code-block:: python

                response = datasets.list()

        Response type:
            list

        Returns:
            Response syntax:
                .. code-block:: python

                    [
                        {
                            "created_at": "2018-03-03T09:04:58.274324",
                            "dataset_id": "1377232365920",
                            "name": "cats-dogs",
                            "organization_id": "1102940376065",
                            "props": {
                                "attributes": [
                                    {
                                        "categories": [
                                            {
                                                "id": 1,
                                                "name": "犬"
                                            },
                                            {
                                                "id": 2,
                                                "name": "猫"
                                            },
                                            {
                                                "id": 3,
                                                "name": "その他"
                                            }
                                        ],
                                        "id": 0,
                                        "name": "cats_dogs"
                                    }
                                ]
                            },
                            "type": "classify",
                            "updated_at": "2018-03-03T09:04:58.274324"
                        },
                        {
                            .....
                            .....
                        },
                        .....
                    ]

        """
        res = self._api.list_datasets(self.organization_id)
        return [Dataset(self._api, **data) for data in res]

    def delete(self, dataset_id):
        """delete a dataset

        Request syntax:
            .. code-block:: python

                response = datasets.delete(dataset_id='1377232365920')

        Params:
            - **dataset_id** (str): dataset id

        Response type:
            dict
        Returns:
            Response syntax:
                .. code-block:: python

                    {
                        "created_at": "2018-04-10T07:49:30.514794",
                        "dataset_id": "1410805969256",
                        "name": "test-dataset",
                        "organization_id": "1102940376065",
                        "props": {
                            "attributes": [
                                {
                                    "categories": [
                                        {
                                            "id": 1,
                                            "name": "犬"
                                        },
                                        {
                                            "id": 2,
                                            "name": "猫"
                                        },
                                        {
                                            "id": 3,
                                            "name": "その他"
                                        }
                                    ],
                                    "id": 0,
                                    "name": "cats_dogs"
                                }
                            ]
                        },
                        "type": "detection",
                        "updated_at": "2018-04-10T07:49:30.514794"
                    }

        """
        res = self._api.delete_dataset(self.organization_id, dataset_id)
        return Dataset(self._api, **res)
