# -*- coding: utf-8 -*-
import copy
from concurrent.futures import ThreadPoolExecutor, as_completed

from abeja.common.config import FETCH_WORKER_COUNT
from abeja.common.file_factory import file_factory
from abeja.common.iterator import Iterator
from abeja.datasets.base import DatasetBase
from abeja.exceptions import InvalidDataFormat


class DatasetItem(DatasetBase):
    """a model class for DatasetItem

    Properties:
        - organization_id (str)
        - dataset_id (str)
        - dataset_item_id (int)
        - attributes (dict)
        - created_at (datetime)
        - updated_at (datetime)
        - source_data (list)
    """

    def __init__(self, api, organization_id, dataset_id, dataset_item_id,
                 attributes=None, source_data=None, created_at=None, updated_at=None):
        """

        :param api: dataset api client
        :param organization_id:
        :param dataset_id:
        :param dataset_item_id:
        :param attributes: meta data annotated to source data.
        :param source_data: (list) list of source data stored in external storage.
        :param created_at:
        :param updated_at:
        """
        self._api = api
        self.organization_id = organization_id
        self.dataset_id = dataset_id
        self.dataset_item_id = dataset_item_id
        self.attributes = attributes
        self.created_at = created_at
        self.updated_at = updated_at
        self.source_data = []

        for item in source_data:
            data_uri, data_type, _source_data = self._parse_source_data(item)
            source_file = file_factory(api, data_uri, data_type, **_source_data)
            self.source_data.append(source_file)

    def __repr__(self):
        return "<{} organization_id:{} " \
               "dataset_id:{} dataset_item_id: {} " \
               "attributes:{} source_data:{}".format(self.__class__.__name__,
                                                     self.organization_id,
                                                     self.dataset_id,
                                                     self.dataset_item_id,
                                                     self.attributes,
                                                     self.source_data)

    def _parse_source_data(self, source_data):
        """extract data_uri and data_type from data

        :param source_data:
        :return: str, str, dict
        :raises:
          - InvalidDataFormat: when data does not includes required keys.
        """
        data = copy.deepcopy(source_data)
        try:
            data_uri = data.pop('data_uri')
            data_type = data.pop('data_type')
        except KeyError as e:
            raise InvalidDataFormat('{} is missing'.format(e))
        return data_uri, data_type, data


def _download_item_content(item: DatasetItem) -> DatasetItem:
    # download content and cache to local disk
    for data in item.source_data:
        data.get_content()
    return item


class DatasetItemIterator(Iterator):
    def __init__(self, api, organization_id, dataset_id,
                 next_page_token=None, limit=None, prefetch=False):
        self._api = api
        self.organization_id = organization_id
        self.dataset_id = dataset_id
        self.next_page_token = next_page_token
        self.limit = limit
        self.prefetch = prefetch
        super().__init__()

    def __iter__(self):
        if self.prefetch:
            return self._items_iter_with_prefetch()
        else:
            return self._items_iter()

    def _items_iter_with_prefetch(self):
        page = self._page()
        with ThreadPoolExecutor(max_workers=FETCH_WORKER_COUNT) as executor:
            futures = []
            while page:
                futures += [
                    executor.submit(_download_item_content, item) for item in page
                ]
                page = self._page()
            for f in as_completed(futures):
                download_item = f.result()
                yield download_item

    def _page(self):
        """get a page of items in dataset"""
        params = {}
        if self.next_page_token:
            params['next_page_token'] = self.next_page_token
        if self.limit:
            params['limit'] = self.limit
        res = self._api.list_dataset_items(self.organization_id, self.dataset_id, params=params)
        self.next_page_token = res.get('next_page_token')
        return [DatasetItem(self._api, self.organization_id, **_item) for _item in res['items']]


class DatasetItems:
    """a class for a dataset item

        .. code-block:: python

            from abeja.datasets import Client

            client = Client()
            dataset = client.get_dataset(dataset_id='1410805969256')
            dataset_items = dataset.dataset_items()

    """

    def __init__(self, api, organization_id, dataset_id):
        self._api = api
        self.organization_id = organization_id
        self.dataset_id = dataset_id

    def create(self, source_data, attributes):
        """create a item in dataset

        Request syntax:
            .. code-block:: python

                source_data = [
                    {
                        'data_type': 'images/jpeg',
                        'data_uri': 'datalake://1200123803688/20170815T044617-f20dde80-1e3b-4496-bc06-1b63b026b872',
                        'height': 500,
                        'width': 200
                    }
                ]

                attributes = {
                    'classification': {
                        'id': 1,
                        'label': '犬'
                    },
                    'custom_format': {
                        'anything': 'something'
                    },
                    'detection': [
                        {
                            'id': 2,
                            'label': '猫',
                            'rect': [795, 118, 1143, 418]
                        },
                    ]
                }

                response = dataset_items.create(source_data=source_data, attributes=attributes)

        Params:
            - **source_data** (list): meta data annotated to source data.
            - **attribute** (dict): list of source data stored in external storage.

        Return type:
            dict

        Returns:
            Response syntax:
                .. code-block:: python

                    {
                        'attributes': {
                            'classification': {
                                'id': 1,
                                'label': '犬'
                            },
                            'custom_format': {
                                'anything': 'something'
                            },
                            'detection': [
                                {
                                    'id': 2,
                                    'label': '猫',
                                    'rect': [795, 118, 1143, 418]
                                },
                            ]
                        },
                        'created_at': '2017-12-27T06:25:00.394026',
                        'dataset_id': '3823146073675',
                        'dataset_item_id': 0,
                        'source_data': [
                            {
                                'data_type': 'images/jpeg',
                                'data_uri': 'datalake://1200123803688/20170815T044617-f20dde80-1e3b-4496-bc06-1b63b026b872',
                                'height': 500,
                                'width': 200
                            }
                        ],
                        'organization_id': '1102940376065',
                        'updated_at': '2017-12-27T06:25:00.394026'
                    }

        """
        res = self._api.create_dataset_item(self.organization_id, self.dataset_id, source_data, attributes)
        # res = self._api.create_dataset_item(self.organization_id, self.dataset_id, source_data, attributes)
        return DatasetItem(self._api, self.organization_id, **res)

    def get(self, dataset_item_id):
        """get a item in dataset

        Request syntax:
            .. code-block:: python

                response = dataset_items.get(dataset_item_id=0)

        Params:
            - **dataset_item_id** (int): dataset item id

        Return type:
            dict

        Returns:
            Response syntax:
                .. code-block:: python

                    {
                        'attributes': {
                            'classification': {
                                'id': 1,
                                'label': '犬'
                            },
                            'custom_format': {
                                'anything': 'something'
                            },
                            'detection': [
                                {
                                    'id': 2,
                                    'label': '猫',
                                    'rect': [795, 118, 1143, 418]
                                },
                            ]
                        },
                        'created_at': '2017-12-27T06:25:00.394026',
                        'dataset_id': '3823146073675',
                        'dataset_item_id': 0,
                        'source_data': [
                            {
                                'data_type': 'images/jpeg',
                                'data_uri': 'datalake://1200123803688/20170815T044617-f20dde80-1e3b-4496-bc06-1b63b026b872',
                                'height': 500,
                                'width': 200
                            }
                        ],
                        'organization_id': '1102940376065',
                        'updated_at': '2017-12-27T06:25:00.394026'
                    }

        """
        res = self._api.get_dataset_item(self.organization_id, self.dataset_id, dataset_item_id)
        return DatasetItem(self._api, self.organization_id, **res)

    def list(self, next_page_token=None, limit=None, prefetch=False):
        """generate all dataset_items in a dataset

        Request syntax:
            .. code-block:: python

                response = dataset_items.list()

        Params:
            - **next_page_token** (str) : next page token to get the next items. **[optional]**
            - **limit** (int): limit of items. **[optional]**
            - **prefetch** :(bool)**[optional]**

        Return type:
            list

        Returns:
            .. code-block:: python

                [
                    {
                        'attributes': {
                            'classification': {
                                'id': 1,
                                'label': '犬'
                            },
                            'custom_format': {
                                'anything': 'something'
                            },
                            'detection': [
                                {
                                    'id': 2,
                                    'label': '猫',
                                    'rect': [795, 118, 1143, 418]
                                },
                            ]
                        },
                        'created_at': '2017-12-27T06:25:00.394026',
                        'dataset_id': '3823146073675',
                        'dataset_item_id': 0,
                        'source_data': [
                            {
                                'data_type': 'images/jpeg',
                                'data_uri': 'datalake://1200123803688/20170815T044617-f20dde80-1e3b-4496-bc06-1b63b026b872',
                                'height': 500,
                                'width': 200
                            }
                        ],
                        'organization_id': '1102940376065',
                        'updated_at': '2017-12-27T06:25:00.394026'
                    },
                    .....
                    .....

                ]
        """
        return DatasetItemIterator(self._api, self.organization_id, self.dataset_id,
                                   next_page_token, limit, prefetch)

    def delete(self, dataset_item_id):
        """Delete a datset item.

        Request syntax:
            .. code-block:: python

                response = dataset_items.delete(dataset_item_id=0)

        Params:
            -**dataset_item_id** (int): dataset itme id

        Responses:
            return the deleted dataset item

            Return syntax:
                .. code-block:: python

                    .. code-block:: python

                    {
                        'attributes': {
                            'classification': {
                                'id': 1,
                                'label': '犬'
                            },
                            'custom_format': {
                                'anything': 'something'
                            },
                            'detection': [
                                {
                                    'id': 2,
                                    'label': '猫',
                                    'rect': [795, 118, 1143, 418]
                                },
                            ]
                        },
                        'created_at': '2017-12-27T06:25:00.394026',
                        'dataset_id': '3823146073675',
                        'dataset_item_id': 0,
                        'source_data': [
                            {
                                'data_type': 'images/jpeg',
                                'data_uri': 'datalake://1200123803688/20170815T044617-f20dde80-1e3b-4496-bc06-1b63b026b872',
                                'height': 500,
                                'width': 200
                            }
                        ],
                        'organization_id': '1102940376065',
                        'updated_at': '2017-12-27T06:25:00.394026'
                    }

        """
        res = self._api.delete_dataset_item(self.organization_id, self.dataset_id, dataset_item_id)
        return DatasetItem(self._api, self.organization_id, **res)
