import os
from logging import getLogger

from abeja.common.file_helpers import extract_zipfile
from abeja.exceptions import BadRequest, Unauthorized, Forbidden, NotFound, MethodNotAllowed, \
    ResourceNotFound
from abeja.train.api.client import APIClient
from abeja.train.statistics import Statistics


# from abejamodelcore.logging import get_logger


class Client:

    ORGANIZATION_ID = os.environ.get('ABEJA_ORGANIZATION_ID')
    JOB_DEFINITION_NAME = os.environ.get('TRAINING_JOB_DEFINITION_NAME')
    TRAINING_JOB_ID = os.environ.get('TRAINING_JOB_ID')

    def __init__(self):
        self.api = APIClient()
        self.logger = getLogger('train-api')

    def download_training_result(self, training_job_id: str, path: str=None) -> None:
        """download training artifact that includes model file or logs.
        Training artifact itself is a zip file, this function extracts it.

        :param training_job_id:
        :return:
        :raises: abeja.exceptions.BadRequest
                 abeja.exceptions.Unauthorized
                 abeja.exceptions.Forbidden
                 abeja.exceptions.NotFound
                 abeja.exceptions.MethodNotAllowed
                 abeja.exceptions.InternalServerError
                 json.JSONDecodeError
                 PermissionError
                 ValueError
                 IOError
        """
        response = self.api.get_training_result(organization_id=Client.ORGANIZATION_ID,
                                                job_definition_name=Client.JOB_DEFINITION_NAME,
                                                training_job_id=training_job_id)
        try:
            artifact_uri = response['artifacts']['complete']['uri']
        except KeyError:
            raise ResourceNotFound(
                'training result for training_id {} is not found'.format(training_job_id))
        content = self._get_content(artifact_uri)
        extract_zipfile(content, path=path)

    def _get_content(self, url: str) -> bytes:
        """get binary content from given url.

        :param url:
        :return:
        :raises: abeja.exceptions.BadRequest
                 abeja.exceptions.Unauthorized
                 abeja.exceptions.Forbidden
                 abeja.exceptions.NotFound
                 abeja.exceptions.MethodNotAllowed
                 abeja.exceptions.InternalServerError
                 json.JSONDecodeError
        """
        res = self.api._connection.request('GET', url)
        return res.content

    def update_statistics(self, statistics: Statistics) -> None:
        """ Notify a job statistics for ABEJA-Platform.

        :param statistics: statistics information
        :return: None
        """
        if not statistics or not statistics.get_statistics():
            self.logger.warning('empty statistics found.')
            return

        try:
            response = self.api.update_statistics(organization_id=Client.ORGANIZATION_ID,
                                                  job_definition_name=Client.JOB_DEFINITION_NAME,
                                                  training_job_id=Client.TRAINING_JOB_ID,
                                                  statistics=statistics.get_statistics())
            self.logger.info('update_statistics result: %s', response)
        except (BadRequest, Unauthorized, Forbidden, NotFound, MethodNotAllowed) as e:
            self.logger.warning('update_statistics result was {}.'.format(str(e)))
        except Exception:
            self.logger.exception('update_statistics result was unexpected error:')
